
%init %{
  // Initialize the numpy module so we can use its C API (otherwise get seg 
  // faults)
  import_array();
  %}

%{
#include "rumd_stats.h"
#include "rumd_rdf.h"
#include "numpy/arrayobject.h"
#include <iostream>
  %}

%include exception.i
%exception {
  try {
    $action
      } catch (RUMD_Error e) {
    std::string errorStr("In class ");
    errorStr += e.className;
    errorStr += ", method ";
    errorStr += e.methodName;
    errorStr += ": ";
    errorStr += e.errorStr;
    SWIG_exception(SWIG_RuntimeError,errorStr.c_str());
  } catch(...) {
    SWIG_exception(SWIG_RuntimeError, "Unknown error");
  }
 }

%include stringTypeMap.i

class Conf {
 public:
  Conf();
  unsigned int GetNumberOfParticles() const;
  void OpenGZ_File(const std::string& filename);
  void CloseCurrentGZ_File();
  void ReadCurrentFile(bool verbose=false);
  ConfigurationMetaData  metaData;
  %extend {
    PyObject* GetPositions() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 3;
      PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
      char *ptr = ((PyArrayObject*) posArray)->data;
      int len = sizeof(float)/sizeof(char);
      for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].x)), len);
	  ptr += len;
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].y)), len);
	  ptr += len;
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].z)), len);
	  ptr += len;
	}
      return posArray;
    } // end GetPositions
    PyObject* GetTypes() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 1;
      PyObject *typeArray = PyArray_SimpleNew(2, dims, NPY_INT);
      char *ptr = ((PyArrayObject*) typeArray)->data;
      int len = sizeof(int)/sizeof(char);
      for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  int type = (int) $self->P[pdx].MyType;
	  memcpy((void*) ptr, (void*)(&type), len);
	  ptr += len;
	}
      return typeArray;
    }
    PyObject* GetImages() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 3;
      PyObject *imArray = PyArray_SimpleNew(2, dims, NPY_INT);
      char *ptr = ((PyArrayObject*) imArray)->data;
      int len = 3*sizeof(int)/sizeof(char);
      for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  int im[3];
	  im[0] = $self->P[pdx].x;
	  im[1] = $self->P[pdx].y;
	  im[2] = $self->P[pdx].z;
	  memcpy((void*) ptr, (void*)(&im), len);
	  ptr += len;
	}
      return imArray;
    }
    PyObject* GetVelocities() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 3;
      PyObject *velArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
      char *ptr = ((PyArrayObject*) velArray)->data;
      int len = sizeof(float)/sizeof(char);
      for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].vx)), len);
	  ptr += len;
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].vy)), len);
	  ptr += len;
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].vz)), len);
	  ptr += len;
	}
      return velArray;
    }
    PyObject* GetForces() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 3;
      PyObject *forArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
      char *ptr = ((PyArrayObject*) forArray)->data;
      int len = sizeof(float)/sizeof(char);
     for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].fx)), len);
	  ptr += len;
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].fy)), len);
	  ptr += len;
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].fz)), len);
	  ptr += len;
	}
      return forArray;
    }
    PyObject* GetPotentialEnergies() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 1;
      PyObject *peArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
      char *ptr = ((PyArrayObject*) peArray)->data;
      int len = sizeof(float)/sizeof(char);
      for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].Ui)), len);
	  ptr += len;
	}
      return peArray;
    }
    PyObject* GetVirials() {
      npy_intp dims[2];
      unsigned int nParticles = $self->GetNumberOfParticles();
      dims[0] = nParticles;
      dims[1] = 1;
      PyObject *virArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
      char *ptr = ((PyArrayObject*) virArray)->data;
      int len = sizeof(float)/sizeof(char);
      for(int pdx = 0; pdx < nParticles; pdx++)
	{
	  memcpy((void*) ptr, (void*)(&($self->P[pdx].Wi)), len);
	  ptr += len;
	}
      return virArray;
    }
  } // end %extend
 };

class rumd_stats {
 public:
  rumd_stats() {}
  void SetVerbose(bool b);
  void SetDirectory(const std::string& directory);
  void ComputeStats(unsigned int first_block=0, int last_block=-1);
  void PrintStats();
  void WriteStats();
  unsigned int GetCount() const;
  std::string GetColumnLabel(unsigned int colIdx);

  %extend {
    PyObject* GetMeanVals() {
      unsigned int nCols = $self->GetNumCols();
      /*
	npy_intp dims = nCols;	
	PyObject *meanValsArray = PyArray_SimpleNew(1, &dims, NPY_DOUBLE);
	char* ptr = ((PyArrayObject*)meanValsArray)->data;
	unsigned int len = nCols*sizeof(double)/sizeof(char);
	const double* mean_vals = $self->GetMeanVals();
	memcpy((void*) ptr, (void*) mean_vals, len);
	return meanValsArray;
      */
      PyObject *meanValsDict = PyDict_New();
      for(unsigned int col = 0; col< nCols;++col) {
	PyDict_SetItem(meanValsDict, PyString_FromString($self->GetColumnLabel(col).c_str()), PyFloat_FromDouble($self->GetMeanVals()[col]));
      }
      return meanValsDict;
      
}
    PyObject* GetMeanSqVals() {
      unsigned int nCols = $self->GetNumCols();

      /*
      npy_intp dims = nCols;
      PyObject *meanValsArray = PyArray_SimpleNew(1, &dims, NPY_DOUBLE);
      char* ptr = ((PyArrayObject*)meanValsArray)->data;
      unsigned int len = nCols*sizeof(double)/sizeof(char);
      const double* meanSq_vals = $self->GetMeanSqVals();
      memcpy((void*) ptr, (void*) meanSq_vals, len);
      return meanValsArray;*/
      
      PyObject *meanSqValsDict = PyDict_New();
      for(unsigned int col = 0; col< nCols;++col) {
	PyDict_SetItem(meanSqValsDict, PyString_FromString($self->GetColumnLabel(col).c_str()), PyFloat_FromDouble($self->GetMeanSqVals()[col]));
      }
      return meanSqValsDict;
      


    }
  }
};


class rumd_rdf {
 public:
  rumd_rdf();
  void SetDirectory(const std::string& directory);
  void ComputeRDF(int nBins, float min_dt, unsigned int first_block=0, int last_block=-1);
  void WriteRDF(const std::string& filename);
  void WriteSDF(const std::string& filename);
  int GetNumTypes() const;
  const double* GetRDF(int type1, int type2);
  const double* GetSDF(int type1, int coord);
  %extend {
    PyObject* GetRDFArray(int type1, int type2) {
      npy_intp nBins = $self->GetNumBins();

      PyObject *rdfArray = PyArray_SimpleNew(1, &nBins, NPY_DOUBLE);
      double* rdfArray_data = (double*) PyArray_DATA(rdfArray);
      const double* rdf = $self->GetRDF(type1, type2);
      for(unsigned int i = 0; i < nBins; ++i)
	rdfArray_data[i] = rdf[i];
      return rdfArray;
    } // end GetRDFArray
    PyObject* GetRadiusValues() {
      npy_intp nBins = $self->GetNumBins();
      PyObject *rValsArray = PyArray_SimpleNew(1, &nBins, NPY_DOUBLE);
      double* rVals_data = (double*) PyArray_DATA(rValsArray);
      const double* rVals = $self->GetRVals();
      for(unsigned int i = 0; i < nBins; ++i)
	rVals_data[i] = rVals[i];
      return rValsArray;
   } // end GetRadiusValues
  } // end % extend
};
