#ifndef INTEGRATORNVT_H
#define INTEGRATORNVT_H

/*
  Copyright (C) 2010  Thomas Schrøder
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  LICENSE.txt file for license details.
*/

#include "rumd/Integrator.h"

class Sample; class SimulationBox;

class IntegratorNVT : public Integrator{
  
 private:
  IntegratorNVT(const IntegratorNVT&);
  IntegratorNVT& operator=(const IntegratorNVT&);

  bool thermostatOn; // NVE or NVT
  
  // Dynamic allocation.
  unsigned int numParticlesSample;  
  
  float thermostatRelaxationTime; // The user chosen relaxation time.
  double targetTemperature; 
  double temperatureChangePerTimeStep;
  float massFactor;

  float*  d_thermostatState; // GPU.
  float*  d_thermostatKineticEnergy; // Array for summation of kinetic energy.
  float4* d_particleMomentum; // Array for summation of total momentum.
  
  void AllocateFromConstructor();
  void FreeArrays();
  void AllocateIntegratorState();
  void UpdateThermostatState();
  
 public:
  IntegratorNVT(float timeStep);
  IntegratorNVT(float timeStep, double targetTemperature);
  ~IntegratorNVT();
  
  // Class methods.
  void Integrate();
  
  // Set methods.
  void SetThermostatOn(bool on = false);
  void SetRelaxationTime(float tau);
  void SetTargetTemperature(double T);
  void SetThermostatState(float Ps);
  void SetMomentumToZero();
  void SetTemperatureChangePerTimeStep(double temperatureChangePerTimeStep);
  
  // Get methods.
  float GetRelaxationTime();
  double GetTargetTemperature();
  float GetThermostatState() const;
  double GetKineticEnergy() const;
  float4 GetTotalMomentum() const;
};

#ifndef SWIG

template <class S> __global__ void integrateNVTAlgorithm( unsigned numParticles, float4* position, float4* velocity, 
							  float4* force, float4* image, S* simulationBox, 
							  float* simulationBoxDevicePointer, bool isCubic, float* thermostatState, 
							  float* thermostatKineticEnergy, float timeStep, bool thermoStatOn );

__global__ void updateNVTThermostatState( float* thermostatKineticEnergy, float* thermostatState, 
					  float thermostatMass, float targetTemperature, 
					  int degreesOfFreedom, float timeStep );

__global__ void calculateMomentum( unsigned int numParticles, float4* velocity, float4* particleMomentum );
__global__ void zeroTotalMomentum( unsigned int numParticles, float4* velocity, float4* particleMomentum );

#endif // SWIG
#endif // INTEGRATORNVT_H
